/*
 *      dxemoji.h
 *
 *      Copyright 2022 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 *
 * Pango
 * pango-emoji-private.h: Emoji handling, private definitions
 *
 * Copyright (C) 2017 Google, Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifndef DXEMOJI_H
#define DXEMOJI_H

#include "dxemojitable.h"

FXbool bsearch_interval(FXwchar c, const struct Interval table[], FXuint n);

#define DEFINEdx_Is_(name) \
static inline FXbool \
dx_Is_##name (FXwchar ch) \
{ \
  return ch >= dx_##name##_table[0].start && \
         bsearch_interval (ch, \
                           dx_##name##_table, \
                           ARRAYNUMBER (dx_##name##_table)); \
}

DEFINEdx_Is_(Emoji)
DEFINEdx_Is_(Emoji_Presentation)
DEFINEdx_Is_(Emoji_Modifier)
DEFINEdx_Is_(Emoji_Modifier_Base)
DEFINEdx_Is_(Extended_Pictographic)

FXbool dx_Is_Emoji_Base_Character(FXwchar ch);
FXbool dx_Is_Emoji_Extended_Pictographic(FXwchar ch);
FXbool dx_Is_Emoji_Emoji_Default(FXwchar ch);
FXbool dx_Is_Emoji_Keycap_Base(FXwchar ch);
FXbool dx_Is_Regional_Indicator(FXwchar ch);

#define kCombiningEnclosingCircleBackslashCharacter 0x20E0
#define kCombiningEnclosingKeycapCharacter 0x20E3
#define kVariationSelector15Character 0xFE0E
#define kVariationSelector16Character 0xFE0F
#define kZeroWidthJoinerCharacter 0x200D

enum DXEmojiScannerCategory {
  EMOJI = 0,
  EMOJI_TEXT_PRESENTATION = 1,
  EMOJI_EMOJI_PRESENTATION = 2,
  EMOJI_MODIFIER_BASE = 3,
  EMOJI_MODIFIER = 4,
  EMOJI_VS_BASE = 5,
  REGIONAL_INDICATOR = 6,
  KEYCAP_BASE = 7,
  COMBINING_ENCLOSING_KEYCAP = 8,
  COMBINING_ENCLOSING_CIRCLE_BACKSLASH = 9,
  ZWJ = 10,
  VS15 = 11,
  VS16 = 12,
  TAG_BASE = 13,
  TAG_SEQUENCE = 14,
  TAG_TERM = 15,
  kMaxEmojiScannerCategory = 16
};

unsigned char dx_EmojiSegmentationCategory(FXwchar codepoint);

typedef unsigned char *emoji_text_iter_t;

/**
 * @brief dx_emoji_end
 * @param text
 * @param n
 * @param isemoji
 * @return number of utf8 chars from text which is emoji
 */
FXint dx_emoji_end(const FXchar *text, FXint n, bool *isemoji);
FXint dx_emoji_end(const FXString &text, bool *isemoji);

#endif  /* DXEMOJI_H */

