/*
 *      emojis.cpp
 *
 *      Copyright 2022 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include <new>
#include "emojis.h"
#include "dxutils.h"

std::unordered_map<std::string,dxEmoji> emojis;

bool isEmoji(const FXString &code)
{
    auto it = emojis.find(dxutils::toStdString(code));
    if(it != emojis.end()) return true;
    //fallback for emojies ended FE0F
    it = emojis.find(dxutils::toStdString(code.left(code.length()-3)));
    if(it != emojis.end()) return true;
    return false;
}

FXIcon* emoji(const FXString &code)
{
    auto it = emojis.find(dxutils::toStdString(code));
    if(it != emojis.end()) return it->second.emoji;
    //fallback for emojies ended FE0F
    it = emojis.find(dxutils::toStdString(code.left(code.length()-3)));
    if(it != emojis.end()) return it->second.emoji;
    return NULL;
}

void drawEmoji(FXDCWindow &dc, const FXString &code, FXint x, FXint y, FXbool useEmojiIcon, FXFont *emojifont, FXFont *ircfont)
{
    if(useEmojiIcon)
    {
        FXint dt = ircfont->getFontDescent();
        auto it = emojis.find(dxutils::toStdString(code));
        if(it != emojis.end())
        {
            if(it->second.emoji) dc.drawIcon(it->second.emoji,x+1,y-it->second.emoji->getHeight()+dt);
            return;
        }
        //fallback for emojies ended FE0F
        it = emojis.find(dxutils::toStdString(code.left(code.length()-3)));
        if(it != emojis.end())
        {
            if(it->second.emoji) dc.drawIcon(it->second.emoji,x+1,y-it->second.emoji->getHeight()+dt);
            return;
        }
    }
    else
    {
        dc.setFont(emojifont);
        FXint sp, sc;
        for(sp = sc = 0; sp < code.length(); sp = code.inc(sp), sc++)
        {
            //NOTE realy needed?
            if(code.wc(sp)!=L'\U0000200D' && code.wc(sp)!=L'\U0000FE0F')
            {
                dc.drawText(x,y,&code[sp],code.inc(sp)-sp);
                x += emojifont->getCharWidth(code.wc(sp));
            }
        }
        dc.setFont(ircfont);
    }
}

FXint emojiWidth(const FXString &code, FXbool useEmojiIcon, FXFont *emojifont)
{
    if(useEmojiIcon)
    {
        auto it = emojis.find(dxutils::toStdString(code));
        if(it != emojis.end())
        {
            if(it->second.emoji) return it->second.emoji->getWidth()+2;
            else return 0;
        }
        //fallback for emojies ended FE0F
        it = emojis.find(dxutils::toStdString(code.left(code.length()-3)));
        if(it != emojis.end())
        {
            if(it->second.emoji) return it->second.emoji->getWidth()+2;
            else return 0;
        }
        return 0;
    }
    else
    {
        FXint w = 0;
        FXint sp, sc;
        for(sp = sc = 0; sp < code.length(); sp = code.inc(sp), sc++)
        {
            //NOTE realy needed?
            if(code.wc(sp)!=L'\U0000200D' && code.wc(sp)!=L'\U0000FE0F') w += emojifont->getCharWidth(code.wc(sp));
        }
        return w;
    }
}

dxStringMap asciiemojis;

dxStringMap shortemojis;
