/*
 *      emojis.h
 *
 *      Copyright 2022 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#ifndef EMOJIS_H
#define EMOJIS_H

#include <string>
#include <unordered_map>
#include <fx.h>
#include <FXDCWindow.h>
#include "defs.h"

struct dxEmoji {
    FXString filename;              //ex. "0039-fe0f-20e3"
    FXString name;                  //ex. "keycap number sign"
    FXString shortname;             //ex. ":hash:"
    FXString category;              //ex. "symbols"
    FXint emoji_order;
    FXIcon *icon;                   //Icon 48x48 for emoji select dialog
    FXIcon *emoji;                  //icon for use on dxtext or dxtextfield
    FXString lowername;

    dxEmoji(FXString f, FXString n, FXString s, FXString c, FXint o, FXIcon *i, FXIcon *e)
        : filename(f), name(n), shortname(s), category(c), emoji_order(o), icon(i), emoji(e), lowername(n.lower())
    {}
    dxEmoji() {}
};

extern std::unordered_map<std::string,dxEmoji> emojis;

bool isEmoji(const FXString &code);

FXIcon* emoji(const FXString &code);

FXint emojiWidth(const FXString &code, FXbool useEmojiIcon, FXFont *emojifont);

void drawEmoji(FXDCWindow &dc, const FXString &code, FXint x, FXint y, FXbool useEmojiIcon, FXFont *emojifont, FXFont *ircfont);

extern dxStringMap asciiemojis;

extern dxStringMap shortemojis;

#endif  /* EMOJIS_H */

