/*
 *      emojisdialog.h
 *
 *      Copyright 2022 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#ifndef EMOJISDIALOG_H
#define	EMOJISDIALOG_H

#include "defs.h"
#include "fxext.h"

class EmojisList;

class EmojiItem : public FXIconItem {
    FXDECLARE(EmojiItem)
    friend class EmojisList;
private:
    EmojiItem(const EmojiItem&);
    EmojiItem& operator=(const EmojiItem&);

    FXString m_name;
    FXString m_shortname;
    FXint m_order;
protected:
    EmojiItem():FXIconItem(){}
    void draw(const FXIconList* list,FXDC& dc,FXint x,FXint y,FXint w,FXint h) const;
    FXint hitItem(const FXIconList* list,FXint rx,FXint ry,FXint rw=1,FXint rh=1) const;
    void drawBigIcon(const FXIconList* list,FXDC& dc,FXint x,FXint y,FXint w,FXint h) const;
public:
    EmojiItem(const FXString& unicode,FXIcon* bi, const FXString &name, const FXString &shortname, FXint order);
    virtual ~EmojiItem();

    /// Return width of item as drawn in list
    FXint getWidth(const FXIconList* list) const;
    /// Return height of item as drawn in list
    FXint getHeight(const FXIconList* list) const;
    const FXString &name() const;
    const FXString &shortname() const;
    FXint order() const;
};

class EmojisList : public FXIconList
{
    FXDECLARE(EmojisList)
protected:
    EmojisList();
private:
    EmojisList(const EmojisList&);
    EmojisList& operator=(const EmojisList&);
public:
    EmojisList(FXComposite *p,FXObject* tgt=NULL,FXSelector sel=0,FXuint opts=ICONLIST_BIG_ICONS|ICONLIST_SINGLESELECT|ICONLIST_COLUMNS|LAYOUT_FILL_X|LAYOUT_FILL_Y,FXint x=0,FXint y=0,FXint w=0,FXint h=0);

    long onQueryTip(FXObject*,FXSelector,void*);
    void sortItems();
};

class EmojisDialog: public FXDialogBox
{
    FXDECLARE(EmojisDialog)
public:
    EmojisDialog(FXApp *app);
    EmojisDialog(FXWindow *window);
    virtual ~EmojisDialog();

    virtual void create();

    long onCancel(FXObject*,FXSelector,void*);
    long onClose(FXObject*,FXSelector,void*);
    long onTableDoubleClick(FXObject*,FXSelector,void*);
    long onToolbar(FXObject*,FXSelector,void*);
    long onKeyPress(FXObject*,FXSelector,void*);
    long onSearch(FXObject*,FXSelector,void*);
    long onSearchFocusin(FXObject*,FXSelector,void*);

    const FXString &unicode() const;
    void setTarget(FXWindow *w);
private:
    EmojisDialog():m_contents(NULL),m_buttonframe(NULL),m_searchframe(NULL),m_smileys(NULL)
    ,m_buttonClose(NULL),m_search(NULL),m_toolbar(NULL),m_people(NULL),m_animals(NULL),m_food(NULL),m_travel(NULL)
    ,m_activities(NULL),m_objects(NULL),m_symbols(NULL),m_flags(NULL),m_filtered(NULL),m_switcher(NULL)
    {}
    EmojisDialog(const EmojisDialog& orig);

    void fillEmojisList();
    void fillFiltered();

    FXVerticalFrame *m_contents;
    FXHorizontalFrame *m_buttonframe, *m_searchframe;
    EmojisList *m_smileys;
    dxEXButton *m_buttonClose;
    FXTextField *m_search;
    FXToolBar *m_toolbar;
    EmojisList *m_people;
    EmojisList *m_animals;
    EmojisList *m_food;
    EmojisList *m_travel;
    EmojisList *m_activities;
    EmojisList *m_objects;
    EmojisList *m_symbols;
    EmojisList *m_flags;
    EmojisList *m_filtered;
    FXString m_unicode;
    FXSwitcher *m_switcher;
    FXWindow *m_target; //target for doubleclick
};

#endif	/* EMOJISDIALOG_H */

