/*
 *      emojisdialog.h
 *
 *      Copyright 2022 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#ifndef EMOJISDIALOG_H
#define EMOJISDIALOG_H

#include <QtGui>
#include <QtWidgets>
#include <QStandardItem>
#include <QItemDelegate>
#include <QPainter>
#include <QSortFilterProxyModel>
#include <QStackedWidget>

class EmojiItem : public QStandardItem
{
public:
    enum EmojiRoles {
        EORDER = Qt::UserRole+2,
        EUNICODE,
        ENAME,
        ESHORTNAME,
        ECATEGORY,
        EFILENAME,
    };

    EmojiItem(const QString &unicode, const QString &filename, const QString &name, const QString &shortname, const QString &category, int order);
};

class EmojiItemDelegate : public QItemDelegate
{
public:
    EmojiItemDelegate();
    QSize sizeHint(const QStyleOptionViewItem& option, const QModelIndex& index) const;
    void paint(QPainter* painter, const QStyleOptionViewItem& option, const QModelIndex& index) const;
};

class EmojiItemSortModel : public QSortFilterProxyModel
{
    Q_OBJECT
public:
    EmojiItemSortModel(const QString &category, QObject *parent = 0);

    void setSearch(const QString &newSearch);
private:
    QString m_search;
    QString m_category;

    bool lessThan(const QModelIndex &left, const QModelIndex &right) const;
    bool filterAcceptsRow(int source_row, const QModelIndex &source_parent) const;
};

class EmojiItemModel : public QStandardItemModel
{
public:
    EmojiItemModel(QObject* parent);

    QString getUnicode(int row) const;
};

class EmojisDialog : public QDialog
{
    Q_OBJECT
public:
    explicit EmojisDialog(QWidget *parent = 0);
private:
    QListView *m_smileys;
    QListView *m_people;
    QListView *m_animals;
    QListView *m_food;
    QListView *m_travel;
    QListView *m_activities;
    QListView *m_objects;
    QListView *m_symbols;
    QListView *m_flags;
    QListView *m_filtered;
    EmojiItemModel *m_model;
    EmojiItemSortModel m_smileysModel;
    EmojiItemSortModel m_peopleModel;
    EmojiItemSortModel m_animalsModel;
    EmojiItemSortModel m_foodModel;
    EmojiItemSortModel m_travelModel;
    EmojiItemSortModel m_activitiesModel;
    EmojiItemSortModel m_objectsModel;
    EmojiItemSortModel m_symbolsModel;
    EmojiItemSortModel m_flagsModel;
    EmojiItemSortModel m_filteredModel;
    QLineEdit *m_search;
    QStackedWidget *m_switch;

    void fillEmojisList();
private slots:
    void accept();
    void slotSmileys();
    void slotPeople();
    void slotAnimals();
    void slotFood();
    void slotTravel();
    void slotActivities();
    void slotObjects();
    void slotSymbols();
    void slotFlags();
    void slotSearch();
    void slotDoubleClick(QModelIndex index);
signals:
    void unicodeClicked(const QString &unicode);
};

#endif // EMOJISDIALOG_H
